'Jennrich Test  for equality of correlation matrices (Jennrich 1970)
'User inputs two group objects
'Output is a Table with the test statistic, p-value and degrees of freedom

logmode +addin

'Command for user to input 2nd group object
%group1 = _this.@name
%group2 =%0

if (@len(%group2)=0) then
	!result = @uidialog("caption", "Jennrich Test", "edit", %group2, "Second group")
	if !result = -1 then
		stop
	endif
endif

'Checking that group sizes are equal
If {%group1}.@count <> {%group2}.@count then
%msg = "Groups must have the same number of series to compute this test"
@uiprompt(%msg)
stop
endif


do {%group1}.cor(out=my1) corr
do {%group2}.cor(out=my2) corr

%r1 = @getnextname("r1")
matrix {%r1} = my1corr
%r2 = @getnextname("r2")
matrix {%r2} = my2corr

!n1 = @rows({%group1})    'defining sample size of group 1
!n2 = @rows({%group2})    'defining sample size of group 2
!pval=0
!chisq=0
!df=0

call calc({%r1}, {%r2}, !n1, !n2, !pval, !chisq, !df)

subroutine local calc(matrix r1, matrix r2, scalar n1, scalar n2, scalar pval, scalar chisq, scalar df)
!csum = n1 * n2 / ( n1 +n2 )
matrix Rbar = ( n1 * R1 + n2 * R2) / ( n1 + n2 )
matrix Rinv = @inverse( Rbar ) 
matrix S = @emult(Rbar,Rinv)	  		'element by element

scalar p = @rows(S)    
scalar q = @columns(S)
matrix S  = S  + @identity(p)  
matrix Sinv = @inverse( S ) 
    
matrix L = @sqr( !csum ) * Rinv * ( R1 - R2 ) 
matrix Linv = @inverse( L )
vector dia = @getmaindiagonal(L)						'Defining diag(x)
matrix LL = L*@transpose(L)     
scalar trL = @trace(LL) 
scalar diasinv =  @transpose(dia) * Sinv *dia

ChiSq = 0.5*trL - diasinv				'Chi-squared test statistic
    
scalar m = @rows(R1)    
scalar n = @columns(R1)
df = n*(n-1)/2							'degrees of freedom
pval = @chisq(ChiSq, df)    		'p-value
endsub

'Results
'Presented in a Table
%tn = @getnextname("_table")
Table(9,2) {%tn}
{%tn}.setwidth(1) 18
{%tn}.setwidth(2) 10
{%tn}.setjust(a1:a9) left
!rowcounter=1
{%tn}(!rowcounter,1) = "Jennrich Test for Equality of Correlation Matrices"
!rowcounter = !rowcounter + 1
%datestring = "Date: " + @datestr(@now,"MM/DD/YY") + "   Time: " + @datestr(@now,"HH:MM")
{%tn}(!rowcounter,1) = %datestring
!rowcounter=!rowcounter+1
%samplestr = "Group 1: "  + @upper(%group1) + ", Observations: " + @str(!n1)
{%tn}(!rowcounter,1) = %samplestr
!rowcounter=!rowcounter+1
%samplestr = "Group 2: " + @upper(%group2) + ", Observations: " + @str(!n2)
{%tn}(!rowcounter,1) = %samplestr
!rowcounter=!rowcounter+1
%samplestr = "Correlation Matrix Size: "  + @str(@columns(my1corr)) + " x " + @str(@rows(my1corr))
{%tn}(!rowcounter,1) = %samplestr
!rowcounter=!rowcounter+1
{%tn}.setlines(!rowcounter) +d
!rowcounter = !rowcounter + 1
'---------------------------
{%tn}(!rowcounter,1) = "Test Statistic: "
{%tn}(!rowcounter,2) = !Chisq
!rowcounter = !rowcounter + 1
{%tn}(!rowcounter,1) = "p-value:"
{%tn}(!rowcounter,2) =  !pval
!rowcounter = !rowcounter + 1
{%tn}(!rowcounter,1) = "df:"
{%tn}(!rowcounter,2) =  !df
!rowcounter = !rowcounter + 1
{%tn}.setlines(!rowcounter) +d
!rowcounter = !rowcounter + 1

_this.display {%tn}
d {%tn}
delete my1corr my2corr {%r1} {%r2}


